#include <linux/config.h>
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/ip.h>
#include <net/icmp.h>
#include <net/ip.h>
#include <net/checksum.h>
struct in_device;
#include <net/route.h>
#include <linux/netfilter_ipv4/ip_tables.h>
#include "ipt_SIGN.h"
#include "ipt_verify.h"

#if 1
#define DEBUGP printk
#else
#define DEBUGP(format, args...)
#endif

/* $Id: ipt_verify1.c,v 1.2 2001/08/29 20:59:45 ahu Exp $ */

static char *staging;
void shaSignPacket(const struct iphdr *ih, const char *key, int offset, int len, char *staging, char *signptr);
static int verify(const struct sk_buff *skb,
			     const struct net_device *in,
			     const struct net_device *out,
			     const void *matchinfo,
			     int offset,
			     const void *hdr,
			     u_int16_t datalen,
			     int *hotdrop)

{
	const struct ipt_verify_info *verify = matchinfo;
	struct iphdr *ih=skb->nh.iph;
	struct spshdr sh;

	unsigned char *signptr;
	unsigned char digest[20];

	int spslen, n;

	if(ih->protocol!=128) /* nope */
		return 0;
	
	if(ntohs(ih->tot_len)<sizeof(sh)+20)
		return 0; 

	/* save SPS header */
	memcpy(&sh,(char*)ih+ih->ihl*4,sizeof(sh));

	spslen=sizeof(sh)+sh.sig_bytes;
	if(ntohs(ih->tot_len)<spslen+20)
		return 0;

	if(sh.algorithm!=1)
		return 0;

	/* should now calculate signature */

	signptr=(char*)ih+ih->ihl*4+sizeof(sh);

	shaSignPacket(ih, verify->secret, spslen, ntohs(ih->tot_len)-ih->ihl*4-spslen, staging, digest); /* SIGN! */

//	memcpy(staging+8,(char *)ih+ih->ihl*4+spslen,ntohs(ih->tot_len)-ih->ihl*4-spslen);

	printk("Calculated signature ('%s'): ", verify->secret);
	for(n=0;n<20;n++)
	  printk("%02x",digest[n]);
	printk("\nIn header: ");
	for(n=0;n<20;n++)
	  printk("%02x",signptr[n]);
	printk("\n");

	/* and compare */
	
	return !memcmp(digest,signptr,20);
}

static int check(const char *tablename,
		 const struct ipt_ip *ip,
		 void *matchinfo,
		 unsigned int matchinfosize,
		 unsigned int hook_mask)
{
/* 	const struct ipt_stripsps_info *stripspsinfo = targinfo; */

 	if (matchinfosize != IPT_ALIGN(sizeof(struct ipt_verify_info))) {
		DEBUGP("verify: matchinfosize %u != %u\n", matchinfosize,
		       IPT_ALIGN(sizeof(struct ipt_verify_info)));
  		return 0;
  	}
	return 1;

	/* Only allow these for packet filtering. */
/*	if (strcmp(tablename, "filter") != 0) {
		DEBUGP("STRIPSPS: bad table `%s'.\n", tablename);
		return 0;
	} */
	
	if ((hook_mask & ~((1 << NF_IP_LOCAL_IN)
			   | (1 << NF_IP_FORWARD)
			   )) != 0) {
		DEBUGP("STRIPSPS: bad hook mask %X\n", hook_mask);
		return 0;
	}

	return 1;
}

static struct ipt_match ipt_verify_reg
= { { NULL, NULL }, "verify", &verify, &check, NULL, THIS_MODULE };


static int __init init(void)
{
	if(!(staging=kmalloc(2000, GFP_KERNEL)))
		return -ENOMEM;

	if (ipt_register_match(&ipt_verify_reg))
		return -EINVAL;
	return 0;
}

static void __exit fini(void)
{
	ipt_unregister_match(&ipt_verify_reg);
}

module_init(init);
module_exit(fini);
